@@ -1,8 +1,11 @@
 import asyncio
 import logging
 
+from carim_discord_bot.rcon import protocol
+
 log = logging.getLogger(__name__)
 tasks = dict()
+splits = dict()
 _sequence_number = 0
 lock = asyncio.Lock()
 default_timeout = 10
@@ -37,9 +40,23 @@ async def register(key, future: asyncio.Future, timeout=default_timeout):
 
 
 async def incoming(key, packet):
-    log.debug(f'incoming key {key}')
-    future = tasks.pop(key)
-    future.set_result(packet)
+    log.debug(f'incoming key {key} and type {type(packet.payload)}')
+
+    if not packet.payload.is_split():
+        future = tasks.pop(key)
+        future.set_result(packet)
+    else:
+        split = splits.get(key, [''] * packet.payload.count)
+        split[packet.payload.index] = packet.payload.data
+
+        if all(split):
+            data = ''.join(split)
+            new_packet = protocol.Packet(protocol.Command(key, data=data))
+            future = tasks.pop(key)
+            future.set_result(new_packet)
+            splits.pop(key)
+        else:
+            splits[key] = split
 
 
 async def wait_for_timeout(key, future, timeout):
